"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Command = void 0;
const chalk_1 = __importDefault(require("chalk"));
const commander_1 = require("commander");
const figlet_1 = __importDefault(require("figlet"));
const constants_1 = require("../constants");
const factories_1 = require("../factories");
const options_1 = require("../options");
const utils_1 = require("../utils");
const urlCommand_1 = require("./urlCommand");
/**
 * Command extension class
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
class Command extends commander_1.Command {
    constructor() {
        super();
        this.actions = [];
        this.commandOptions = {};
    }
    initialize(initializeAction) {
        initializeAction();
        return this;
    }
    optionInteractive(optionConfig) {
        const { shortName, longName, description, defaultValue, configKey, agileServiceProviders, repoServiceProviders, prompt, choices, } = optionConfig;
        const variableName = this.getVariableName(longName);
        const decoratedDescription = description + this.getDecoratedDescription(optionConfig);
        this.option(`${shortName}, ${longName} <${variableName}>`, decoratedDescription);
        // Add action for getting option value
        return this.addAction((serviceCollection) => __awaiter(this, void 0, void 0, function* () {
            const { inputService } = serviceCollection;
            const currentValue = this.opts()[variableName];
            // If defined, means they've passed in an argument
            if (currentValue !== undefined) {
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                this.commandOptions[variableName] = currentValue;
                return;
            }
            if (configKey) {
                // Try to get value from configuration - return if exists
                const configuredVal = utils_1.Config.getValueWithDefault(configKey);
                if (configuredVal) {
                    // eslint-disable-next-line @typescript-eslint/no-explicit-any
                    this.commandOptions[variableName] = configuredVal;
                    return;
                }
            }
            const agileServiceProvider = this.getAgileProvider();
            const repoServiceProvider = this.getRepoProvider();
            // Check that current agile provider is included in list of option service providers
            if (agileServiceProviders && agileServiceProvider && !agileServiceProviders.includes(agileServiceProvider)) {
                return;
            }
            // Check that current repo provider is included in list of option service providers
            if (repoServiceProviders && repoServiceProvider && !repoServiceProviders.includes(repoServiceProvider)) {
                return;
            }
            const questionPrompt = prompt || `Enter a value for ${longName}:`;
            const optionChoices = typeof choices === "function" ? yield choices(serviceCollection) : choices;
            // Ask user for value - use multiple choice if applicable
            const answeredVal = optionChoices
                ? yield inputService.multiChoiceQuestion(questionPrompt, optionChoices)
                : yield inputService.askQuestion(questionPrompt, defaultValue);
            if (!(answeredVal === null || answeredVal === void 0 ? void 0 : answeredVal.trim().length)) {
                return;
            }
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            this.commandOptions[variableName] = answeredVal;
            // Ask user if they'd like to save to .env if configKey exists to save it
            if (configKey && (yield inputService.confirmAction("Would you like to save to your local environment file?"))) {
                const { contentService, logger } = serviceCollection;
                const currentEnvFile = yield contentService.getIfExists(constants_1.FileConstants.envFileName);
                const updatedEnvFile = utils_1.Config.getUpdatedEnvFileContents(configKey, answeredVal, currentEnvFile);
                yield contentService.set(constants_1.FileConstants.envFileName, updatedEnvFile);
                logger.log(`Saved to ${constants_1.FileConstants.envFileName}`);
            }
        }));
    }
    /**
     * Gets the name of the command
     *
     * @returns {string} Name of individual command (not full command path)
     */
    getName() {
        return this._name;
    }
    /**
     * Sets parent command
     *
     * @param {Command} parentCommand Parent command
     * @returns {Command} this
     */
    setParent(parentCommand) {
        this.parentCommand = parentCommand;
        return this;
    }
    /**
     * Gets the parent command
     *
     * @returns {Command} Parent command
     */
    getParent() {
        return this.parentCommand;
    }
    /**
     * Gets command path, delimited by '/'
     *
     * @returns {string} Full command path name (e.g. grandparent/parent/child)
     */
    getCommandPath() {
        const getCommandPath = (command) => {
            const parent = command.getParent();
            if (!parent) {
                return command.getName();
            }
            return `${getCommandPath(parent)}/${command.getName()}`;
        };
        return getCommandPath(this);
    }
    /**
     * Adds a sub command and sets this as parent command
     *
     * @param {Command} subCommand Sub command to add
     * @returns {Command} this
     */
    addSubCommand(subCommand) {
        this.addCommand(subCommand);
        subCommand.setParent(this);
        return this;
    }
    getDecoratedDescription(optionConfig) {
        const { description, configKey, agileServiceProviders, repoServiceProviders, choices } = optionConfig;
        return (description +
            (configKey ? `\nCan be provided via environment variable ${utils_1.Config.getEnvironmentVariableName(configKey)}` : "") +
            "\nCan be provided interactively by user if not available" +
            (choices && typeof choices !== "function" ? `\nOptions: (${choices.join(", ")})` : "") +
            (agileServiceProviders ? `\nOnly valid for agile providers: (${agileServiceProviders.join(", ")})` : "") +
            (repoServiceProviders ? `\nOnly valid for repo providers: (${repoServiceProviders.join(", ")})` : ""));
    }
    getAgileProvider() {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return this.commandOptions.agileProvider;
    }
    getRepoProvider() {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return this.commandOptions.repoProvider;
    }
    /**
     * Gets a variable name from a long option name
     *
     * @param {string} longName Long variable name `--my-variable-name`
     * @returns {string} camelCasedVariableName (e.g. myVariableName)
     */
    getVariableName(longName) {
        const longNameSplit = longName.replace(/^--/, "").split("-");
        if (longNameSplit.length === 1) {
            return longNameSplit[0];
        }
        const capitalizeFirstLetter = (word) => {
            const firstChar = word.charAt(0);
            return word.replace(firstChar, firstChar.toUpperCase());
        };
        return longNameSplit.map((value, index) => (index > 0 ? capitalizeFirstLetter(value) : value)).join("");
    }
    addAgileProviderOptions() {
        return this.optionInteractive(options_1.agileProviderInteractiveOption)
            .optionInteractive(options_1.agileBaseUrlInteractiveOption)
            .optionInteractive(options_1.agileAccessTokenInteractiveOption)
            .optionInteractive(options_1.agileProjectNameInteractiveOption);
    }
    addRepoProviderOptions() {
        return this.optionInteractive(options_1.repoProviderInteractiveOption)
            .optionInteractive(options_1.repoBaseUrlInteractiveOption)
            .optionInteractive(options_1.repoAccessTokenInteractiveOption)
            .optionInteractive(options_1.repoProjectNameInteractiveOption);
    }
    addPlaybookOptions() {
        return this.optionInteractive(options_1.playbookOptionalAccessTokenInteractiveOption);
    }
    addAction(actionHandler) {
        this.actions.push(actionHandler);
        this.action(() => __awaiter(this, void 0, void 0, function* () {
            var _a, _b, _c;
            const serviceCollection = this.serviceCollection || factories_1.ServiceCollectionFactory.create();
            this.commandOptions = Object.assign(Object.assign({}, this.opts()), this.commandOptions);
            const commandProperties = {
                command: this.getCommandPath(),
            };
            // Add command path to default properties for observability
            serviceCollection.logger.addDefaultProperties(commandProperties);
            (_a = serviceCollection.metrics) === null || _a === void 0 ? void 0 : _a.addDefaultProperties(commandProperties);
            (_b = serviceCollection.metrics) === null || _b === void 0 ? void 0 : _b.increment(constants_1.MetricNames.CommandExecution);
            for (const action of this.actions) {
                yield action(serviceCollection, this.commandOptions);
            }
            // Flush all metrics to app insights
            (_c = serviceCollection.metrics) === null || _c === void 0 ? void 0 : _c.flush();
        }));
        return this;
    }
    printHelp() {
        this.addAction((serviceCollection) => {
            serviceCollection.logger.log(this.helpInformation());
        });
        return this;
    }
    asciiArt(message) {
        this.addAction((serviceCollection) => {
            serviceCollection.logger.log(chalk_1.default.cyanBright(figlet_1.default.textSync(message)));
        });
        return this;
    }
    addLinkCommands(links) {
        links.forEach((link) => this.addSubCommand(urlCommand_1.urlCommand(link)));
        return this;
    }
    addPhoneTree() {
        this.addAction((serviceCollection) => __awaiter(this, void 0, void 0, function* () {
            const { inputService } = serviceCollection;
            const subCommandNames = this.commands.map((command) => `${command.name()} - ${command.description()}`);
            const subCommandNameChosen = (yield inputService.multiChoiceQuestion(this.name(), subCommandNames)).split(" - ")[0];
            const subCommandChosen = this.commands.find((command) => command.name() === subCommandNameChosen);
            if (subCommandChosen) {
                yield subCommandChosen.parseAsync(this.args);
            }
        }));
        return this;
    }
    toMarkdownDoc(parentCommandPath) {
        const commandPath = `${parentCommandPath ? parentCommandPath + " " : ""}${this.name()}`;
        let doc = `## \`${commandPath}\`\n\n`;
        doc += "```\n" + this.helpInformation() + "```";
        for (const command of this.commands) {
            if (command instanceof Command) {
                doc += `\n${command.toMarkdownDoc(commandPath)}`;
            }
        }
        return doc;
    }
    toTableOfContents(linkPrefix, parentIndentation, parentCommandPath) {
        const indentation = parentIndentation !== undefined ? parentIndentation + "  " : "";
        const commandPath = `${parentCommandPath ? parentCommandPath + " " : ""}${this.name()}`;
        const line = "- [`" +
            commandPath +
            "`]" +
            `(${linkPrefix ? linkPrefix : ""}#${commandPath.replace(/ /g, "-")}) - ${this.description()}`;
        let toc = indentation + line;
        for (const command of this.commands) {
            if (command instanceof Command) {
                toc += `\n${command.toTableOfContents(linkPrefix, indentation, commandPath)}`;
            }
        }
        return toc;
    }
    setServiceCollection(serviceCollection) {
        if (serviceCollection) {
            this.serviceCollection = serviceCollection;
        }
        return this;
    }
}
exports.Command = Command;
