"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GitHubAgileService = void 0;
const constants_1 = require("../../../../constants");
const models_1 = require("../../../../models");
const utils_1 = require("../../../../utils");
const shared_1 = require("../../../shared");
const baseAgileService_1 = require("../../baseAgileService");
/**
 * Performing agile-related business logic on GitHub platform
 */
class GitHubAgileService extends baseAgileService_1.BaseAgileService {
    constructor(options, inputService, logger) {
        super(options, inputService, logger);
        // Backlog Items
        this.getBacklogItems = (ids) => __awaiter(this, void 0, void 0, function* () {
            if (ids) {
                return Promise.all(ids.map((id) => this.getBacklogItem(+id)));
            }
            const issues = yield this.github.getGithubIssues();
            return Promise.all(issues.map((issue) => this.mapGitHubIssueToBacklogItem(issue)));
        });
        this.createBacklogItems = (items) => {
            return Promise.all(items.map((item) => this.createBacklogItem(item)));
        };
        this.deleteBacklogItems = () => __awaiter(this, void 0, void 0, function* () {
            throw new Error("Deleting issues is not supported by GitHub");
        });
        // Project
        this.createProject = () => {
            throw new Error("Not yet implemented");
        };
        this.getProject = () => {
            throw new Error("Not yet implemented");
        };
        this.deleteProject = () => {
            throw new Error("Not yet implemented");
        };
        // Sprints
        this.getSprint = (id) => __awaiter(this, void 0, void 0, function* () {
            const milestone = yield this.github.getMilestone(+id);
            return this.mapMilestoneToSprint(milestone);
        });
        this.createProviderSprints = (sprints) => {
            return Promise.all(sprints.map((sprint) => this.createSprint(sprint)));
        };
        this.deleteSprint = (id) => __awaiter(this, void 0, void 0, function* () {
            return this.github.deleteMilestone(+id);
        });
        this.github = new shared_1.GitHubRestService(options);
        this.logger = logger;
    }
    getBacklogItem(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const issue = yield this.github.getGitHubIssue(id);
            return this.mapGitHubIssueToBacklogItem(issue);
        });
    }
    /**
     * Create a backlog item. Not all are going to be issues.
     *
     * Epic = Label
     * Feature = Project
     * Story, Task, Bug = Issue
     *
     * If tasks/bugs are children of a story, they will be included within the body of
     * the story's issue.
     *
     * @param {BacklogItem} backlogItem Backlog item to create
     * @param {GitHubLabel} label Label associated with backlog item
     * @returns {Promise<BacklogItem>} Newly created Backlog Item
     */
    createBacklogItem(backlogItem, label) {
        return __awaiter(this, void 0, void 0, function* () {
            switch (backlogItem.type) {
                case models_1.BacklogItemType.Epic:
                    return this.createGitHubEpic(backlogItem);
                case models_1.BacklogItemType.Feature:
                    return this.createGitHubFeature(backlogItem, label);
                case models_1.BacklogItemType.Story:
                case models_1.BacklogItemType.Task:
                case models_1.BacklogItemType.Bug:
                    return this.createGitHubIssue(backlogItem, label);
            }
        });
    }
    /**
     * GitHub has no concept of "epics." We are using "labels" as a way of grouping
     * issues in an epic-category.
     *
     * @param {BacklogItem} backlogItem Epic to create
     * @returns {Promise<BacklogItem>} Newly created "epic" (GitHub label)
     */
    createGitHubEpic(backlogItem) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.log(`GitHub doesn't natively have a concept of "epics".
    The closest parity is to create a "label" which will allow you to filter on all child issues`);
            const { name, description, children } = backlogItem;
            const labelName = this.createLabelName(name);
            const label = yield this.github.getOrCreateGitHubLabel(labelName, description);
            return {
                id: label.id.toString(),
                name: labelName,
                type: models_1.BacklogItemType.Epic,
                children: yield this.createChildren(children, label),
                description,
            };
        });
    }
    createLabelName(epicName) {
        return epicName.toLowerCase().replace(/[^a-zA-Z0-9]+/g, "-");
    }
    /**
     * GitHub has no concept of "features." We are using "projects" as a way of grouping
     * issues in a feature-category. This seems to be a common practice among large
     * open-source projects on GitHub.
     *
     * @param {BacklogItem} backlogItem Feature-type backlog item
     * @param {GitHubLabel|undefined} label GitHub Label
     * @returns {Promise<BacklogItem>} Newly created "feature" (GitHub project)
     */
    createGitHubFeature(backlogItem, label) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.log(`GitHub doesn't natively have a concept of "features".
    The closest parity is to create a "project" which will provide a view for all child issues`);
            const { name, description, children } = backlogItem;
            const projectId = yield this.github.getOrCreateGitHubProject(name, description);
            const toDoColumnId = yield this.createColumns(projectId);
            const createdChildren = yield this.createChildren(children, label);
            yield this.assignChildrenToColumn(createdChildren, toDoColumnId);
            return {
                id: projectId.toString(),
                name,
                description,
                type: models_1.BacklogItemType.Feature,
                children: createdChildren,
            };
        });
    }
    createColumns(projectId) {
        return __awaiter(this, void 0, void 0, function* () {
            const columns = utils_1.Config.getValue(constants_1.ConfigKey.AgileBoardColumns);
            const firstColumn = utils_1.Config.getValue(constants_1.ConfigKey.AgileBoardFirstColumn);
            let firstColumnId = undefined;
            for (const col of columns) {
                const createdCol = yield this.github.getOrCreateProjectColumn(projectId, col);
                if (col === firstColumn) {
                    firstColumnId = createdCol;
                }
            }
            return firstColumnId;
        });
    }
    createChildren(children, label) {
        return __awaiter(this, void 0, void 0, function* () {
            return children
                ? Promise.all(children.map((child) => this.createBacklogItem(child, label)))
                : undefined;
        });
    }
    assignChildrenToColumn(children, columnId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (children && columnId) {
                for (const child of children) {
                    yield this.github.addIssueToProjectColumn(columnId, child.metadata.db_id);
                }
            }
        });
    }
    createGitHubIssue(backlogItem, label) {
        return __awaiter(this, void 0, void 0, function* () {
            const { name, description, acceptanceCriteria, children } = backlogItem;
            const body = this.createIssueBody(description, acceptanceCriteria, children);
            // Create issue in GitHub
            const issue = yield this.github.createGitHubIssue(name, body, label);
            // Create backlog item from issue
            return this.mapGitHubIssueToBacklogItem(issue);
        });
    }
    mapGitHubIssueToBacklogItem(issue) {
        const { title, number, node_id, body } = issue;
        const nodeIdBuffer = Buffer.from(node_id, "base64");
        const nodeIdDecoded = nodeIdBuffer.toString("ascii").toLowerCase();
        const db_id = +nodeIdDecoded.substring(nodeIdDecoded.indexOf("issue") + 5);
        return {
            name: title,
            type: models_1.BacklogItemType.Story,
            id: `${number}`,
            description: body,
            metadata: {
                node_id,
                db_id,
            },
        };
    }
    createIssueBody(description, acceptanceCriteria, children) {
        return [
            "## Description",
            description,
            "## Acceptance Criteria",
            this.stringifyAcceptanceCriteria(acceptanceCriteria),
            "## Tasks",
            this.stringifyChildTasks(children),
        ].join("\n");
    }
    stringifyAcceptanceCriteria(acceptanceCriteria) {
        return acceptanceCriteria ? this.createChecklist(acceptanceCriteria) : "";
    }
    stringifyChildTasks(children) {
        return children ? this.createChecklist(children.map((child) => child.name)) : "";
    }
    createChecklist(items) {
        return items ? items.map((item) => `- [ ] ${item}`).join("\n") : "";
    }
    createSprint(sprint) {
        return __awaiter(this, void 0, void 0, function* () {
            const { name, finishDate } = sprint;
            const milestone = yield this.github.createOrUpdateMilestone(name, finishDate);
            return this.mapMilestoneToSprint(milestone);
        });
    }
    mapMilestoneToSprint(milestone) {
        const { id, title, due_on, number } = milestone;
        return {
            name: title,
            id: id.toString(),
            finishDate: due_on ? new Date(Date.parse(due_on)) : undefined,
            metadata: {
                milestoneNumber: number,
            },
        };
    }
}
exports.GitHubAgileService = GitHubAgileService;
