"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectService = void 0;
const yaml_1 = __importDefault(require("yaml"));
/**
 * A service following [the template pattern]{@link https://en.wikipedia.org/wiki/Template_method_pattern}
 * for object storage and retrieval.
 *
 * This decouples storage and interpolation from object retrieval, so commands
 * can delegate those responsibilities and focus on commanding.
 */
class ObjectService {
    /**
     * Creates an instance of an ObjectService.
     *
     * @param {StorageService} storageService The storage service to back the
     * ObjectService with.
     * @returns {ObjectService<T>} An ObjectService instance.
     * @class
     * @template T
     */
    constructor(storageService) {
        this.storageService = storageService;
    }
    /**
     * Gets an object by name, and optionally by location.
     *
     * @param {string} name The name of the object.
     * @param {string} location The location of the object. If null, some default
     * should be used.
     * @returns {Promise<T>} The object, if found.
     * @throws If the object cannot be read from that location.
     * @template T
     */
    get(name, location) {
        return __awaiter(this, void 0, void 0, function* () {
            const content = yield this.read(name, location);
            if (!content) {
                throw new Error("Failed to read content.");
            }
            const interpolated = this.interpolate(content);
            return this.parse(interpolated, name);
        });
    }
    /**
     * Gets an existing object by name, and optionally by location.
     *
     * @param {string} name The name of the object.
     * @param {string} location The location of the object. If null, some default
     * should be used.
     *
     * @returns {Promise<T>} The object, if found: Null otherwise.
     * @template T
     */
    getIfExists(name, location) {
        return __awaiter(this, void 0, void 0, function* () {
            const content = yield this.read(name, location);
            if (!content) {
                return undefined;
            }
            const interpolated = this.interpolate(content);
            return this.parse(interpolated, name);
        });
    }
    /**
     * Sets an object by name, and optionally by location.
     *
     * @param {string} name The name of the object to store.
     * @param {T} obj The instance of the object to store.
     * @param {string} location The place to store the object. If null, some
     * default should be used.
     *
     * @template T
     */
    set(name, obj, location) {
        return __awaiter(this, void 0, void 0, function* () {
            const content = typeof obj === "string" ? obj : this.stringify(obj, name);
            yield this.write(name, content, location);
        });
    }
    /**
     * Parses an object. The default implementation is backed by JSON.parse or yaml.parse
     * If this method is used with classes and not just DTOs, this method should be
     * overridden.
     *
     * @param {string} content The JSON object to parse.
     * @param {string} fileName The name of the file to parse
     * @returns {T} The parsed object.
     * @template T
     */
    parse(content, fileName) {
        // In some cases, content is already parsed
        if (typeof content !== "string") {
            return content;
        }
        /* eslint-disable prettier/prettier */
        const obj = this.isYamlFile(fileName)
            ? yaml_1.default.parse(content)
            : this.isJsonFile(fileName)
                ? JSON.parse(content)
                : content;
        /* eslint-enable prettier/prettier */
        if (!obj) {
            throw new Error(`Failed to parse ${content}.`);
        }
        return obj;
    }
    /**
     * List all files in a location
     *
     * @param {string} location Location to list files
     * @returns {Promise<string[]>} List of files
     */
    list(location) {
        return this.storageService.list(location);
    }
    /**
     * A protected method to find an object. If storage is tree-like, for example,
     * the definition of find might recursively search towards the root.
     *
     * @param {string} name The name of the object to find.
     * @param {string} startingLocation The location to start in storage. If null,
     * the search starts at the default location.
     *
     * @returns {Promise<string | undefined>} The object in JSON format if found:
     * Otherwise, undefined.
     */
    find(name, startingLocation) {
        return this.storageService.find(name, startingLocation);
    }
    /**
     * A protected method to read an object.
     *
     * @param {string} name The name of the object to read.
     * @param {string} location The location to read in storage. If null, reads
     * from the default location.
     *
     * @returns {Promise<string | undefined>} The object in JSON format if an
     * object of that name exists in that location: Otherwise, undefined.
     */
    read(name, location) {
        return this.storageService.read(name, location);
    }
    /**
     * A protected method to write an object.
     *
     * @param {string} name The name of the object to write.
     * @param {string} content The serialized object to write.
     * @param {string} location The location to write in storage. If null, writes
     * to the default location.
     *
     * @returns {Promise<boolean>} Whether the write was successful.
     */
    write(name, content, location) {
        return this.storageService.write(name, content, location);
    }
    /**
     * A virtual method to interpolate variables into some provided content.
     *
     * @param {string} content Content with variables defined.
     *
     * @returns {string} The content with variables replaced.
     */
    interpolate(content) {
        return content;
    }
    /**
     * Serializes a DTO.
     *
     * @param {T} content A DTO.
     * @param {string} fileName Name of file
     * @returns {string} The content turned into a JSON string.
     * @template T The type to stringify
     */
    stringify(content, fileName) {
        return this.isYamlFile(fileName) ? yaml_1.default.stringify(content) : JSON.stringify(content, null, 4);
    }
    isYamlFile(fileName) {
        return fileName.endsWith(".yml") || fileName.endsWith(".yaml");
    }
    isJsonFile(fileName) {
        return fileName.endsWith(".json");
    }
}
exports.ObjectService = ObjectService;
