"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GitHubRestService = void 0;
const rest_1 = require("@octokit/rest");
const utils_1 = require("../../utils");
class GitHubRestService {
    constructor(options) {
        const { accessToken } = options;
        const { repo, owner } = utils_1.RegexExtractor.getGitHubInfo(options);
        this.options = {
            owner,
            // Required to use with GitHub enterprise https://octokit.github.io/rest.js/v18
            baseUrl: "https://api.github.com",
            repo,
        };
        this.octokit = new rest_1.Octokit({
            userAgent: "projector",
            auth: accessToken,
        });
    }
    getOrCreateGitHubLabel(name, description, color) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.octokit.issues.getLabel(Object.assign(Object.assign({}, this.options), { name: name }));
                return response.data;
            }
            catch (err) {
                // Label does not yet exist
                if (err.status === 404) {
                    const response = yield this.octokit.issues.createLabel(Object.assign(Object.assign({}, this.options), { name, color: color || utils_1.Utils.randomColor(), description }));
                    return response.data;
                }
                else {
                    throw err;
                }
            }
        });
    }
    getOrCreateGitHubProject(projectName, description) {
        return __awaiter(this, void 0, void 0, function* () {
            const repoProjectsResponse = yield this.octokit.projects.listForRepo(Object.assign({}, this.options));
            const project = repoProjectsResponse.data.find((repo) => repo.name === projectName);
            if (project) {
                return project.id;
            }
            const createProjectResponse = yield this.octokit.projects.createForRepo(Object.assign(Object.assign({}, this.options), { name: projectName, body: description }));
            return createProjectResponse.data.id;
        });
    }
    getOrCreateProjectColumn(projectId, name) {
        return __awaiter(this, void 0, void 0, function* () {
            const columnsListResponse = yield this.octokit.projects.listColumns({
                project_id: projectId,
            });
            const column = columnsListResponse.data.find((col) => col.name === name);
            if (column) {
                return column.id;
            }
            const createdCol = yield this.octokit.projects.createColumn({
                project_id: projectId,
                name,
            });
            return createdCol.data.id;
        });
    }
    addIssueToProjectColumn(columnId, dbId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.octokit.projects.createCard({
                column_id: columnId,
                content_id: dbId,
                content_type: "Issue",
            });
        });
    }
    getGitHubIssue(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield this.octokit.issues.get(Object.assign(Object.assign({}, this.options), { issue_number: id }));
            return response.data;
        });
    }
    getGithubIssues() {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield this.octokit.issues.list(Object.assign({}, this.options));
            return response.data;
        });
    }
    createGitHubIssue(title, body, label) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield this.octokit.issues.create(Object.assign(Object.assign({}, this.options), { title,
                body, 
                // Github's API types don't match up exactly
                // In some places, label.description can be undefined, in other places it can't.
                // To mitigate, we take the stance of not letting it be undefined (See our GitHubLabel interface definition)
                // and then just casting as any in this instance
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                labels: label ? [label] : undefined }));
            return response.data;
        });
    }
    closeGitHubIssue(id) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.octokit.issues.update(Object.assign(Object.assign({}, this.options), { issue_number: id, state: "closed" }));
        });
    }
    getMilestone(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield this.octokit.issues.getMilestone(Object.assign(Object.assign({}, this.options), { milestone_number: id }));
            return response.data;
        });
    }
    createOrUpdateMilestone(name, dueDate) {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO Perf improvement here - shouldn't need to make request every single time
            const response = yield this.octokit.issues.listMilestones(Object.assign({}, this.options));
            const milestone = response.data.find((m) => m.title === name);
            if (milestone) {
                const updatedResponse = yield this.octokit.issues.updateMilestone(Object.assign(Object.assign({}, this.options), { milestone_number: milestone.number, due_on: dueDate === null || dueDate === void 0 ? void 0 : dueDate.toISOString() }));
                return updatedResponse.data;
            }
            else {
                const createdMilestoneResponse = yield this.octokit.issues.createMilestone(Object.assign(Object.assign({}, this.options), { title: name, due_on: dueDate === null || dueDate === void 0 ? void 0 : dueDate.toISOString() }));
                return createdMilestoneResponse.data;
            }
        });
    }
    deleteMilestone(milestoneNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.octokit.issues.deleteMilestone(Object.assign(Object.assign({}, this.options), { milestone_number: milestoneNumber }));
        });
    }
}
exports.GitHubRestService = GitHubRestService;
